@@ -57,6 +57,8 @@ static list<scim_bridge_imcontext_id_t> free_imcontexts;
 
 static BackEndPointer scim_backend = NULL;
 
+static IMEngineFactoryPointer fallback_imengine_factory = NULL;
+
 static IMEngineInstancePointer shared_imengine = NULL;
 static IMEngineInstancePointer fallback_imengine = NULL;
 
@@ -64,7 +66,7 @@ static String scim_language = "";
 
 static bool imengine_shared = false;
 
-static bool enabled_by_default = true;
+static bool enabled_by_default = false;
 
 static unsigned int imengine_id = 0;
 
@@ -107,7 +109,7 @@ class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
 
         static void slot_fallback_commit (IMEngineInstanceBase *imengine, const WideString &commit_string);
         static void slot_forward_key_event (IMEngineInstanceBase *imengine, const KeyEvent &key_event);
-        
+
         ScimBridgeAgentIMContextImpl (ScimBridgeAgentClientListener *new_client_listener);
         ~ScimBridgeAgentIMContextImpl ();
 
@@ -214,7 +216,7 @@ void ScimBridgeAgentIMContext::static_initialize (ScimBridgeAgentPanelListenerPr
 {
     scim_backend = backend;
 
-    IMEngineFactoryPointer fallback_imengine_factory = scim_backend->get_factory (SCIM_COMPOSE_KEY_FACTORY_UUID);
+    fallback_imengine_factory = scim_backend->get_factory (SCIM_COMPOSE_KEY_FACTORY_UUID);
     if (fallback_imengine_factory.null ()) fallback_imengine_factory = new DummyIMEngineFactory ();
 
     fallback_imengine = fallback_imengine_factory->create_instance (String ("UTF-8"), 0);
@@ -235,6 +237,10 @@ void ScimBridgeAgentIMContext::static_finalize ()
 
     shared_imengine = NULL;
     fallback_imengine = NULL;
+
+    fallback_imengine_factory = NULL;
+
+    scim_backend = NULL;
 }
 
 
@@ -279,8 +285,6 @@ preedit_shown (false), preedit_cursor_position (0)
     free_imcontexts.pop_back ();
     imcontexts[imcontext_id] = this;
     id = imcontext_id;
-
-//    if (!imengine_shared) alloc_imengine ();
 }
 
 
@@ -293,18 +297,25 @@ ScimBridgeAgentIMContextImpl::~ScimBridgeAgentIMContextImpl ()
 
 void ScimBridgeAgentIMContextImpl::free ()
 {
-    focus_out ();
+    if (get_imengine () != NULL) focus_out ();
     free_partially ();
 }
 
 
 void ScimBridgeAgentIMContextImpl::free_partially ()
 {
-    panel_listener->remove_input_context (id);
+    panel_listener->remove_input_context ();
+
+    preedit_string.clear ();
+    preedit_cursor_position = 0;
+    preedit_shown = false;
+    preedit_attributes.clear ();
+
     if (imengine_shared && static_cast<ScimBridgeAgentIMContextImpl*> (shared_imengine->get_frontend_data ()) == this) {
         shared_imengine->reset ();
         shared_imengine->set_frontend_data (NULL);
-    } else {
+    } else if (imengine != NULL) {
+        imengine->reset ();
         free_imengine ();
     }
 
@@ -357,12 +368,10 @@ ScimBridgeAgentClientListener *ScimBridgeAgentIMContextImpl::get_client_listener
 
 void ScimBridgeAgentIMContextImpl::reset ()
 {
-    preedit_string.clear ();
-    preedit_cursor_position = 0;
-    preedit_shown = false;
-    preedit_attributes.clear ();
-
-    get_imengine ()->reset ();
+    if (get_imengine () != NULL) {
+        ScimBridgeAgentIMContext* focused_imcontext = static_cast<ScimBridgeAgentIMContext*> (get_imengine ()->get_frontend_data ());
+        if (focused_imcontext == this) get_imengine ()->reset ();
+    }
 }
 
 
@@ -387,7 +396,6 @@ bool ScimBridgeAgentIMContext::is_imengine_shared ()
 void ScimBridgeAgentIMContext::set_imengine_shared (bool shared)
 {
     imengine_shared = shared;
-    shared_imengine = NULL;
 
     if (imengine_shared) {
         IMEngineFactoryPointer factory = scim_backend->get_default_factory (scim_language, "UTF-8");
@@ -406,7 +414,7 @@ void ScimBridgeAgentIMContext::set_imengine_shared (bool shared)
 
             if (imcontext != NULL) {
                 imcontext->free_imengine ();
-                panel_listener->register_input_context (imcontext->get_id (), shared_imengine->get_factory_uuid ());
+                panel_listener->register_input_context (shared_imengine->get_factory_uuid ());
             }
         }
     } else {
@@ -441,12 +449,13 @@ bool ScimBridgeAgentIMContextImpl::is_enabled ()
 void ScimBridgeAgentIMContextImpl::set_enabled (bool new_state)
 {
     enabled = new_state;
-    
+
     if (enabled) {
         IMEngineFactoryPointer factory = scim_backend->get_factory (get_imengine ()->get_factory_uuid ());
+        if (factory.null ()) factory = fallback_imengine_factory;
         PanelFactoryInfo info = PanelFactoryInfo (factory->get_uuid (), utf8_wcstombs (factory->get_name ()), factory->get_language (), factory->get_icon_file ());
         panel_listener->update_factory_info (info);
-        
+
         panel_listener->hide_preedit ();
         panel_listener->hide_aux_string ();
         panel_listener->hide_lookup_table ();
@@ -456,6 +465,8 @@ void ScimBridgeAgentIMContextImpl::set_enabled (bool new_state)
     } else {
         get_imengine ()->focus_out ();
 
+        hide_preedit ();
+
         PanelFactoryInfo info = PanelFactoryInfo (String (""), String (_ ("English/Keyboard")), String ("C"), String (SCIM_KEYBOARD_ICON_FILE));
         panel_listener->update_factory_info (info);
 
@@ -489,7 +500,7 @@ void ScimBridgeAgentIMContextImpl::alloc_imengine ()
         imengine = fallback_imengine;
     } else {
         attach_imengine (imengine);
-        panel_listener->register_input_context (id, imengine->get_factory_uuid ());
+        panel_listener->register_input_context (imengine->get_factory_uuid ());
     }
 
     enabled = false;
@@ -522,12 +533,12 @@ bool ScimBridgeAgentIMContextImpl::filter_key_event (const KeyEvent &key_event)
 {
     IMEngineInstancePointer imengine = get_imengine ();
     if (!imengine.null ()) {
-        ScimBridgeAgentIMContextImpl *focused_imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
+        ScimBridgeAgentIMContext *focused_imcontext = static_cast<ScimBridgeAgentIMContext*> (imengine->get_frontend_data ());
         if (focused_imcontext != this) focus_in ();
     } else {
         focus_in ();
     }
- 
+
     if (!is_enabled ()) {
         return false;
     } else if (get_imengine ()->process_key_event (key_event)) {
@@ -561,24 +572,28 @@ void ScimBridgeAgentIMContextImpl::open_imengine (IMEngineFactoryPointer factory
 {
     focus_out ();
 
+    hide_preedit ();
+    preedit_string.clear ();
+    preedit_attributes.clear ();
+
     if (!factory.null ()) {
-        if (get_imengine ()->get_factory_uuid () == factory->get_uuid ()) {
-            set_enabled (true);
-            if (imengine_shared) set_enabled_by_default (true);
-            return;
-        }
-        
-        IMEngineInstancePointer new_imengine = factory->create_instance ("UTF-8", get_imengine ()->get_id ());
+        set_enabled (true);
+        if (imengine_shared) set_enabled_by_default (true);
+        if (get_imengine ()->get_factory_uuid () == factory->get_uuid ()) return;
+
+        IMEngineInstancePointer new_imengine = factory->create_instance ("UTF-8", imengine_id);
+        ++imengine_id;
+
         if (imengine_shared) {
             shared_imengine = new_imengine;
-            shared_imengine->set_frontend_data (this);
+            shared_imengine->set_frontend_data (static_cast<ScimBridgeAgentIMContext*> (this));
         } else {
             imengine = new_imengine;
-            imengine->set_frontend_data (this);
+            imengine->set_frontend_data (static_cast<ScimBridgeAgentIMContext*> (this));
         }
         attach_imengine (new_imengine);
         scim_backend->set_default_factory (scim_language, factory->get_uuid ());
-        panel_listener->register_input_context (id, factory->get_uuid ());
+        panel_listener->register_input_context (factory->get_uuid ());
         focus_in ();
     } else {
         set_enabled (false);
@@ -595,33 +610,35 @@ void ScimBridgeAgentIMContextImpl::beep ()
 
 void ScimBridgeAgentIMContextImpl::focus_in ()
 {
-    if (imengine_shared) {
-        panel_listener->register_input_context (id, shared_imengine->get_factory_uuid ());
-        ScimBridgeAgentIMContextImpl *focused_imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (shared_imengine->get_frontend_data ());
-        if (focused_imcontext != NULL && focused_imcontext != this) shared_imengine->focus_out ();
+    if (!imengine_shared && imengine == NULL) alloc_imengine ();
 
+    ScimBridgeAgentIMContext *focused_imcontext = static_cast<ScimBridgeAgentIMContext*> (get_imengine ()->get_frontend_data ());
+    if (focused_imcontext != NULL) focused_imcontext->focus_out ();
+
+    get_imengine ()->set_frontend_data (static_cast<ScimBridgeAgentIMContext*> (this));
+    panel_listener->register_input_context (get_imengine ()->get_factory_uuid ());
+
+    if (imengine_shared) {
         if (enabled_by_default) {
             enabled = true;
         } else {
             enabled = false;
         }
-    } else if (imengine == NULL) {
-        alloc_imengine ();
     }
-    get_imengine ()->set_frontend_data (this);
 
     panel_listener->focus_in (get_imengine ()->get_factory_uuid ());
+    panel_listener->update_screen ();
     panel_listener->update_cursor_location (cursor_x, cursor_y);
 
     PanelFactoryInfo info;
     if (enabled) {
         IMEngineFactoryPointer factory = scim_backend->get_factory (get_imengine ()->get_factory_uuid ());
+        if (factory.null ()) factory = fallback_imengine_factory;
         info = PanelFactoryInfo (factory->get_uuid (), utf8_wcstombs (factory->get_name ()), factory->get_language (), factory->get_icon_file ());
     } else {
         info = PanelFactoryInfo (String (""), String (_ ("English/Keyboard")), String ("C"), String (SCIM_KEYBOARD_ICON_FILE));
     }
     panel_listener->update_factory_info (info);
-    panel_listener->update_screen ();
 
     preedit_shown = false;
 
@@ -640,12 +657,21 @@ void ScimBridgeAgentIMContextImpl::focus_in ()
 
 void ScimBridgeAgentIMContextImpl::focus_out ()
 {
-    get_imengine ()->set_frontend_data (this);
+    ScimBridgeAgentIMContext *focused_imcontext = static_cast<ScimBridgeAgentIMContext*> (get_imengine ()->get_frontend_data ());
+
+    get_imengine ()->set_frontend_data (static_cast<ScimBridgeAgentIMContext*> (this));
+    hide_preedit ();
+
+    get_imengine ()->focus_out ();
     if (imengine_shared) shared_imengine->reset ();
     panel_listener->turn_off ();
     panel_listener->focus_out ();
-    get_imengine ()->focus_out ();
-    get_imengine ()->set_frontend_data (NULL);
+
+    if (imengine_shared) {
+        if (focused_imcontext != this) get_imengine ()->set_frontend_data (static_cast<ScimBridgeAgentIMContext*> (focused_imcontext));
+    } else {
+        imengine->set_frontend_data (NULL);
+    }
 }
 
 
@@ -676,13 +702,13 @@ void ScimBridgeAgentIMContextImpl::hide_preedit ()
 {
     if (is_on_the_spot_enabled ()) {
         if (preedit_shown) {
-            preedit_shown = false;
-            preedit_cursor_position = 0;
-            preedit_string.clear ();
-            preedit_attributes.clear ();
             client_listener->push_event (new ScimBridgeAgentSetPreeditShownEvent (id, false));
             client_listener->push_event (new ScimBridgeAgentUpdatePreeditEvent (id));
         }
+        preedit_shown = false;
+        preedit_cursor_position = 0;
+        preedit_string.clear ();
+        preedit_attributes.clear ();
     } else {
         panel_listener->hide_preedit ();
     }
@@ -746,6 +772,8 @@ void ScimBridgeAgentIMContextImpl::update_preedit_caret (int cursor_pos)
 
 void ScimBridgeAgentIMContextImpl::update_preedit_string (const WideString &str, const AttributeList &attrs)
 {
+    if (preedit_string.empty () && str.empty ()) return;
+
     preedit_string = str;
     preedit_attributes = attrs;
 
@@ -754,8 +782,10 @@ void ScimBridgeAgentIMContextImpl::update_preedit_string (const WideString &str,
             client_listener->push_event (new ScimBridgeAgentSetPreeditShownEvent (id, true));
             preedit_shown = true;
         }
+        //preedit_cursor_position = str.length ();
         client_listener->push_event (new ScimBridgeAgentSetPreeditStringEvent (id, preedit_string));
         client_listener->push_event (new ScimBridgeAgentSetPreeditAttributesEvent (id, preedit_attributes));
+        //client_listener->push_event (new ScimBridgeAgentSetPreeditCursorPositionEvent (id, preedit_cursor_position));
         client_listener->push_event (new ScimBridgeAgentUpdatePreeditEvent (id));
     } else {
         panel_listener->set_preedit_string (str, attrs);
@@ -954,6 +984,7 @@ void ScimBridgeAgentIMContextImpl::panel_request_help ()
     help += String (_ ("\n\n"));
 
     IMEngineFactoryPointer factory = scim_backend->get_factory (get_imengine ()->get_factory_uuid ());
+    if (factory.null ()) factory = fallback_imengine_factory;
     help += utf8_wcstombs (factory->get_name ());
     help += String (_ (":\n\n"));
 
