@@ -47,6 +47,8 @@
 
 #include "event/scim-bridge-agent-panel-pending-event.h"
 
+#include "utils/scim-bridge-agent-panel-client.h"
+
 using std::endl;
 using std::ifstream;
 using std::ofstream;
@@ -54,6 +56,16 @@ using std::vector;
 
 using namespace scim;
 
+/* Type def */
+typedef enum _thread_status_t
+{
+    THREAD_UNINITIALIZED,
+    THREAD_INITIALIZING,
+    THREAD_INITIALIZED,
+    THREAD_FINALIZING,
+    THREAD_FINALIZED
+} thread_status_t;
+
 /* Class definition */
 class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
 {
@@ -67,14 +79,16 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
         void prepare (scim_bridge_imcontext_id_t imcontext_id);
         void send ();
 
-		retval_t launch ();
+        retval_t launch ();
+
+        retval_t check_lockfile ();
+        retval_t create_lockfile (pid_t pid);
 
         /* The following functions are virtually protected */
         void initialize ();
 
-        retval_t create_lockfile ();
         retval_t destroy_lockfile ();
-        retval_t update_lockfile ();
+        retval_t update_lockfile (pid_t pid);
 
         void focus_in (const String &factory_uuid);
         void focus_out ();
@@ -109,17 +123,20 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
         void show_factory_menu (const vector<PanelFactoryInfo> &menu);
         void show_help (const String &string);
 
-        void register_input_context (scim_bridge_imcontext_id_t id, const String &factory_uuid);
-        void remove_input_context (scim_bridge_imcontext_id_t id);
+        void register_input_context (const String &factory_uuid);
+        void remove_input_context ();
 
         /* The following functions are virtually private */
         int get_panel_fd ();
 
+        void connect_panel_client ();
+
         void panel_error_occured ();
         void panel_event_occured ();
+        void panel_listener_launched ();
         void panel_listener_exited ();
+        bool panel_listener_is_exit_triggered ();
         void read_event ();
-        bool is_active ();
 
     protected:
 
@@ -129,13 +146,10 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
 
         ScimBridgeAgentProtected *agent;
 
-        String scim_config_name;
-
-        ScimBridgeDisplay *display;
-
-        PanelClient scim_panel_client;
+        ScimBridgeAgentPanelClient *scim_panel_client;
 
         int panel_fd;
+        thread_status_t thread_status;
 
         scim_bridge_imcontext_id_t prepared_imcontext_id;
 
@@ -143,9 +157,7 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
 
         sem_t semaphore;
 
-		bool launched;
-
-        void connect_panel_client ();
+        bool launched;
 
         /* Panel related callbacks */
         void slot_reload_config (scim_bridge_imcontext_id_t id);
@@ -171,11 +183,19 @@ void *run_panel_listener (void *arg)
 {
     ScimBridgeAgentPanelListenerImpl *panel_listener = static_cast<ScimBridgeAgentPanelListenerImpl*> (arg);
 
+    panel_listener->panel_listener_launched ();
+
     timeval previous_time;
     gettimeofday (&previous_time, NULL);
 
-    while (panel_listener->is_active ()) {
-        int panel_fd = panel_listener->get_panel_fd ();
+    while (!panel_listener->panel_listener_is_exit_triggered ()) {
+        const int panel_fd = panel_listener->get_panel_fd ();
+
+        if (panel_fd < 0) {
+            panel_listener->connect_panel_client ();
+            sleep (1);
+            continue;
+        }
 
         timeval timeout;
         timeout.tv_sec = 60 * 60;
@@ -194,11 +214,11 @@ void *run_panel_listener (void *arg)
         gettimeofday (&current_time, NULL);
 
         if ((current_time.tv_sec - previous_time.tv_sec) + (current_time.tv_usec - previous_time.tv_usec) / 1000000 >= 60 * 60) {
-            panel_listener->update_lockfile ();
+            panel_listener->update_lockfile (-1);
             previous_time = current_time;
         }
 
-        if (ret > 0 && panel_listener->is_active ()) {
+        if (ret > 0 && !panel_listener->panel_listener_is_exit_triggered ()) {
             if (FD_ISSET (panel_fd, &error_set)) {
                 panel_listener->panel_error_occured ();
             } else {
@@ -208,6 +228,7 @@ void *run_panel_listener (void *arg)
     }
 
     panel_listener->panel_listener_exited ();
+
     return NULL;
 }
 
@@ -216,7 +237,7 @@ void *run_panel_listener (void *arg)
 ScimBridgeAgentPanelListener *ScimBridgeAgentPanelListener::alloc (const String &config_name, const ScimBridgeDisplay *new_display, ScimBridgeAgentProtected *new_agent)
 {
     ScimBridgeAgentPanelListenerImpl *panel_listener = new ScimBridgeAgentPanelListenerImpl (config_name, new_display, new_agent);
-    if (panel_listener->create_lockfile ()) {
+    if (panel_listener->check_lockfile ()) {
         delete panel_listener;
         return NULL;
     } else {
@@ -226,11 +247,9 @@ ScimBridgeAgentPanelListener *ScimBridgeAgentPanelListener::alloc (const String
 }
 
 
-ScimBridgeAgentPanelListenerImpl::ScimBridgeAgentPanelListenerImpl (const String &config_name, const ScimBridgeDisplay *new_display, ScimBridgeAgentProtected *new_agent):
-agent (new_agent), scim_config_name (config_name), panel_fd (-1), prepared_imcontext_id (-1), launched (false)
+ScimBridgeAgentPanelListenerImpl::ScimBridgeAgentPanelListenerImpl (const String &config_name, const ScimBridgeDisplay *display, ScimBridgeAgentProtected *new_agent): agent (new_agent), panel_fd (-1), thread_status (THREAD_UNINITIALIZED), prepared_imcontext_id (-1), launched (false)
 {
-    display = scim_bridge_alloc_display ();
-    scim_bridge_copy_display (display, new_display);
+    scim_panel_client = ScimBridgeAgentPanelClient::alloc (config_name, display);
 
     sem_init (&semaphore, 0, 0);
 }
@@ -238,7 +257,8 @@ agent (new_agent), scim_config_name (config_name), panel_fd (-1), prepared_imcon
 
 ScimBridgeAgentPanelListenerImpl::~ScimBridgeAgentPanelListenerImpl ()
 {
-    scim_bridge_free_display (display);
+    delete scim_panel_client;
+
     sem_destroy (&semaphore);
 }
 
@@ -247,44 +267,49 @@ void ScimBridgeAgentPanelListenerImpl::initialize ()
 {
     scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 4, "Initializing panel listener...");
 
-    scim_panel_client.signal_connect_reload_config (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_reload_config));
-    scim_panel_client.signal_connect_exit (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_exit));
-    scim_panel_client.signal_connect_update_lookup_table_page_size (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_update_lookup_table_page_size));
-    scim_panel_client.signal_connect_lookup_table_page_up (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_lookup_table_page_up));
-    scim_panel_client.signal_connect_lookup_table_page_down (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_lookup_table_page_down));
-    scim_panel_client.signal_connect_trigger_property (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_trigger_property));
-    scim_panel_client.signal_connect_process_helper_event (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_process_helper_event));
-    scim_panel_client.signal_connect_move_preedit_caret (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_move_preedit_caret));
-    scim_panel_client.signal_connect_select_candidate (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_select_candidate));
-    scim_panel_client.signal_connect_process_key_event (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_process_key_event));
-    scim_panel_client.signal_connect_commit_string (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_commit));
-    scim_panel_client.signal_connect_forward_key_event (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_forward_key_event));
-    scim_panel_client.signal_connect_request_help (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_request_help));
-    scim_panel_client.signal_connect_request_factory_menu (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_request_factory_menu));
-    scim_panel_client.signal_connect_change_factory (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_change_factory));
+    scim_panel_client->signal_connect_reload_config (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_reload_config));
+    scim_panel_client->signal_connect_exit (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_exit));
+    scim_panel_client->signal_connect_update_lookup_table_page_size (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_update_lookup_table_page_size));
+    scim_panel_client->signal_connect_lookup_table_page_up (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_lookup_table_page_up));
+    scim_panel_client->signal_connect_lookup_table_page_down (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_lookup_table_page_down));
+    scim_panel_client->signal_connect_trigger_property (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_trigger_property));
+    scim_panel_client->signal_connect_process_helper_event (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_process_helper_event));
+    scim_panel_client->signal_connect_move_preedit_caret (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_move_preedit_caret));
+    scim_panel_client->signal_connect_select_candidate (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_select_candidate));
+    scim_panel_client->signal_connect_process_key_event (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_process_key_event));
+    scim_panel_client->signal_connect_commit_string (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_commit));
+    scim_panel_client->signal_connect_forward_key_event (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_forward_key_event));
+    scim_panel_client->signal_connect_request_help (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_request_help));
+    scim_panel_client->signal_connect_request_factory_menu (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_request_factory_menu));
+    scim_panel_client->signal_connect_change_factory (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_change_factory));
 }
 
 
 retval_t ScimBridgeAgentPanelListenerImpl::launch ()
 {
-	assert (!launched);
-	launched = true;
+    assert (!launched);
+    launched = true;
 
-    connect_panel_client ();
-    
-	if (pthread_create (&panel_listener_thread, NULL, &run_panel_listener, this)) {
+    thread_status = THREAD_INITIALIZING;
+
+    if (pthread_create (&panel_listener_thread, NULL, &run_panel_listener, this)) {
         scim_bridge_psyslogln (SYSLOG_CRITICAL, "Cannot invoke the panel listener thread");
         return RETVAL_FAILED;
     } else {
-		return RETVAL_SUCCEEDED;
-	}
+        return RETVAL_SUCCEEDED;
+    }
 }
 
 
-retval_t ScimBridgeAgentPanelListenerImpl::create_lockfile ()
+void ScimBridgeAgentPanelListenerImpl::panel_listener_launched ()
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Creating the lockfile...");
+    thread_status = THREAD_INITIALIZED;
+}
 
+
+retval_t ScimBridgeAgentPanelListenerImpl::check_lockfile ()
+{
+    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 6, "Checking the lockfile...");
     const char *lockfile_path = scim_bridge_path_get_lockfile ();
     ifstream input_stream (lockfile_path);
 
@@ -296,12 +321,20 @@ retval_t ScimBridgeAgentPanelListenerImpl::create_lockfile ()
     if (input_stream.good ()) {
         scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Pinging for the process: pid = %d", pid);
         if (kill (pid, 0) == 0) {
-            scim_bridge_psyslogln (SYSLOG_CRITICAL, "Another agent is running. I'll exit.");
+            scim_bridge_println ("Another agent is running...");
             return RETVAL_FAILED;
         }
     }
 
-    if (update_lockfile ()) {
+    return RETVAL_SUCCEEDED;
+}
+
+
+retval_t ScimBridgeAgentPanelListenerImpl::create_lockfile (pid_t pid)
+{
+    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Creating the lockfile...");
+
+    if (update_lockfile (pid)) {
         return RETVAL_FAILED;
     } else {
         return RETVAL_SUCCEEDED;
@@ -309,14 +342,17 @@ retval_t ScimBridgeAgentPanelListenerImpl::create_lockfile ()
 }
 
 
-retval_t ScimBridgeAgentPanelListenerImpl::update_lockfile ()
+retval_t ScimBridgeAgentPanelListenerImpl::update_lockfile (pid_t pid)
 {
     scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 1, "Updating the lockfile...");
 
     const char *lockfile_path = scim_bridge_path_get_lockfile ();
     ofstream output_stream (lockfile_path);
 
-    output_stream << getpid () << endl;
+    pid_t pid0 = pid;
+    if (pid < 0) pid0 = getpid ();
+
+    output_stream << pid0 << endl;
     output_stream.close ();
 
     if (!output_stream.good ()) {
@@ -342,21 +378,14 @@ retval_t ScimBridgeAgentPanelListenerImpl::destroy_lockfile ()
 void ScimBridgeAgentPanelListenerImpl::panel_listener_exited ()
 {
     scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Panel listener exited");
-    panel_fd = -2;
+    thread_status = THREAD_FINALIZED;
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::connect_panel_client ()
 {
-    const char *display_name = scim_bridge_display_get_name (display);
-    if (display_name == NULL) {
-        scim_bridge_psyslogln (SYSLOG_ERROR, "Cannot get the variable, 'DISPLAY'");
-        panel_fd = -1;
-    }
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 3, "'DISPLAY' = '%s'", display_name);
-
-    if (scim_panel_client.open_connection (scim_config_name, String (display_name)) >= 0) {
-        panel_fd = scim_panel_client.get_connection_number ();
+    if (scim_panel_client->open_connection () >= 0) {
+        panel_fd = scim_panel_client->get_connection_number ();
         scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 3, "Panel fd = %d", panel_fd);
     } else {
         scim_bridge_psyslogln (SYSLOG_ERROR, "Failed to open the panel socket");
@@ -370,10 +399,9 @@ void ScimBridgeAgentPanelListenerImpl::panel_error_occured ()
     scim_bridge_psyslogln (SYSLOG_WARNING, "Panel client has been downed...");
 
     shutdown (panel_fd, SHUT_RDWR);
-    scim_panel_client.close_connection ();
+    scim_panel_client->close_connection ();
     panel_fd = -1;
-
-    connect_panel_client ();
+    prepared_imcontext_id = -1;
 }
 
 
@@ -386,6 +414,11 @@ void ScimBridgeAgentPanelListenerImpl::panel_event_occured ()
 
 void ScimBridgeAgentPanelListenerImpl::read_and_dispatch ()
 {
+    if (panel_fd < 0) {
+        scim_bridge_perrorln ("Connection lost with the panel client");
+        return;
+    }
+
     fd_set read_set;
     FD_ZERO (&read_set);
     FD_SET (panel_fd, &read_set);
@@ -395,7 +428,7 @@ void ScimBridgeAgentPanelListenerImpl::read_and_dispatch ()
     timeout.tv_usec = 0;
 
     if (select (panel_fd + 1, &read_set, NULL, NULL, &timeout) > 0) {
-        if (!scim_panel_client.filter_event ()) panel_error_occured ();
+        if (scim_panel_client->filter_event ()) panel_error_occured ();
     }
     sem_post (&semaphore);
 }
@@ -407,27 +440,30 @@ int ScimBridgeAgentPanelListenerImpl::get_panel_fd ()
 }
 
 
-bool ScimBridgeAgentPanelListenerImpl::is_active ()
+bool ScimBridgeAgentPanelListenerImpl::panel_listener_is_exit_triggered ()
 {
-    return get_event_client_id () != -2;
+    return thread_status != THREAD_INITIALIZING && thread_status != THREAD_INITIALIZED;
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::do_close_event_client ()
 {
-    const int tmp_fd = panel_fd;
-    panel_fd = -1;
-    shutdown (tmp_fd, SHUT_RDWR);
-    scim_panel_client.close_connection ();
+    while (thread_status == THREAD_INITIALIZING) usleep (10);
 
-    while (panel_fd != -2) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "Killing panel listener...");
+    if (thread_status == THREAD_INITIALIZED) {
+        thread_status = THREAD_FINALIZING;
+        shutdown (panel_fd, SHUT_RDWR);
+        scim_panel_client->close_connection ();
+
+        while (thread_status != THREAD_FINALIZED) {
+            scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "Killing panel listener...");
+            sem_post (&semaphore);
+            pthread_kill (panel_listener_thread, SIGNOTIFY);
+            usleep (50);
+        }
         sem_post (&semaphore);
-        pthread_kill (panel_listener_thread, SIGNOTIFY);
-        usleep (50);
+        pthread_join (panel_listener_thread, NULL);
     }
-    sem_post (&semaphore);
-    pthread_join (panel_listener_thread, NULL);
 
     destroy_lockfile ();
 }
@@ -436,13 +472,18 @@ void ScimBridgeAgentPanelListenerImpl::do_close_event_client ()
 void ScimBridgeAgentPanelListenerImpl::prepare (scim_bridge_imcontext_id_t imcontext_id)
 {
     scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 5, "panel prepare");
+    if (panel_fd < 0) {
+        scim_bridge_perrorln ("Connection lost with the panel client");
+        return;
+    }
+
     if (prepared_imcontext_id != -1) {
         scim_bridge_perrorln ("Panel client has been already prepared");
         return;
     }
 
     prepared_imcontext_id = imcontext_id;
-    scim_panel_client.prepare (prepared_imcontext_id);
+    scim_panel_client->prepare (prepared_imcontext_id);
 }
 
 
@@ -452,7 +493,7 @@ void ScimBridgeAgentPanelListenerImpl::send ()
     if (prepared_imcontext_id == -1) {
         scim_bridge_perrorln ("Panel client has not yet been prepared");
     } else {
-        scim_panel_client.send ();
+        scim_panel_client->send ();
         prepared_imcontext_id = -1;
     }
 }
@@ -464,7 +505,7 @@ void ScimBridgeAgentPanelListenerImpl::focus_in (const String &factory_uuid)
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.focus_in (prepared_imcontext_id, factory_uuid);
+    scim_panel_client->focus_in (factory_uuid);
 }
 
 
@@ -474,7 +515,7 @@ void ScimBridgeAgentPanelListenerImpl::focus_out ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.focus_out (prepared_imcontext_id);
+    scim_panel_client->focus_out ();
 }
 
 
@@ -484,7 +525,7 @@ void ScimBridgeAgentPanelListenerImpl::update_screen ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_screen (prepared_imcontext_id, scim_bridge_display_get_screen_number (display));
+    scim_panel_client->update_screen ();
 }
 
 
@@ -494,7 +535,7 @@ void ScimBridgeAgentPanelListenerImpl::update_cursor_location (int x, int y)
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_spot_location (prepared_imcontext_id, x, y);
+    scim_panel_client->update_spot_location (x, y);
 }
 
 
@@ -504,7 +545,7 @@ void ScimBridgeAgentPanelListenerImpl::update_factory_info (const PanelFactoryIn
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_factory_info (prepared_imcontext_id, factory_info);
+    scim_panel_client->update_factory_info (factory_info);
 }
 
 
@@ -514,7 +555,7 @@ void ScimBridgeAgentPanelListenerImpl::turn_on ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.turn_on (prepared_imcontext_id);
+    scim_panel_client->turn_on ();
 }
 
 
@@ -524,7 +565,7 @@ void ScimBridgeAgentPanelListenerImpl::turn_off ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.turn_off (prepared_imcontext_id);
+    scim_panel_client->turn_off ();
 }
 
 
@@ -534,7 +575,7 @@ void ScimBridgeAgentPanelListenerImpl::show_aux_string ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.show_aux_string (prepared_imcontext_id);
+    scim_panel_client->show_aux_string ();
 }
 
 
@@ -544,7 +585,7 @@ void ScimBridgeAgentPanelListenerImpl::hide_aux_string ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.hide_aux_string (prepared_imcontext_id);
+    scim_panel_client->hide_aux_string ();
 }
 
 
@@ -554,7 +595,7 @@ void ScimBridgeAgentPanelListenerImpl::set_aux_string (const WideString &str, co
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_aux_string (prepared_imcontext_id, str, attrs);
+    scim_panel_client->update_aux_string (str, attrs);
 }
 
 
@@ -564,7 +605,7 @@ void ScimBridgeAgentPanelListenerImpl::set_lookup_table (const LookupTable &tabl
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_lookup_table (prepared_imcontext_id, table);
+    scim_panel_client->update_lookup_table (table);
 }
 
 
@@ -574,7 +615,7 @@ void ScimBridgeAgentPanelListenerImpl::show_lookup_table ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.show_lookup_table (prepared_imcontext_id);
+    scim_panel_client->show_lookup_table ();
 }
 
 
@@ -584,7 +625,7 @@ void ScimBridgeAgentPanelListenerImpl::hide_lookup_table ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.hide_lookup_table (prepared_imcontext_id);
+    scim_panel_client->hide_lookup_table ();
 }
 
 
@@ -594,7 +635,7 @@ void ScimBridgeAgentPanelListenerImpl::set_preedit_cursor_position (int cursor_p
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_preedit_caret (prepared_imcontext_id, cursor_pos);
+    scim_panel_client->update_preedit_caret (cursor_pos);
 }
 
 
@@ -604,7 +645,7 @@ void ScimBridgeAgentPanelListenerImpl::set_preedit_string (const WideString &str
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_preedit_string (prepared_imcontext_id, str, attrs);
+    scim_panel_client->update_preedit_string (str, attrs);
 }
 
 
@@ -614,7 +655,7 @@ void ScimBridgeAgentPanelListenerImpl::show_preedit ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.show_preedit_string (prepared_imcontext_id);
+    scim_panel_client->show_preedit_string ();
 }
 
 
@@ -624,7 +665,7 @@ void ScimBridgeAgentPanelListenerImpl::hide_preedit ()
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.hide_preedit_string (prepared_imcontext_id);
+    scim_panel_client->hide_preedit_string ();
 }
 
 
@@ -634,7 +675,7 @@ void ScimBridgeAgentPanelListenerImpl::register_properties (const PropertyList &
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.register_properties (prepared_imcontext_id, properties);
+    scim_panel_client->register_properties (properties);
 }
 
 
@@ -644,7 +685,7 @@ void ScimBridgeAgentPanelListenerImpl::update_property (const Property &property
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.update_property (prepared_imcontext_id, property);
+    scim_panel_client->update_property (property);
 }
 
 
@@ -654,7 +695,7 @@ void ScimBridgeAgentPanelListenerImpl::start_helper (const String &helper_uuid)
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.start_helper (prepared_imcontext_id, helper_uuid);
+    scim_panel_client->start_helper (helper_uuid);
 }
 
 
@@ -664,7 +705,7 @@ void ScimBridgeAgentPanelListenerImpl::stop_helper (const String &helper_uuid)
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.stop_helper (prepared_imcontext_id, helper_uuid);
+    scim_panel_client->stop_helper (helper_uuid);
 }
 
 
@@ -674,7 +715,7 @@ void ScimBridgeAgentPanelListenerImpl::send_helper_event (const String &helper_u
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.send_helper_event (prepared_imcontext_id, helper_uuid, trans);
+    scim_panel_client->send_helper_event (helper_uuid, trans);
 }
 
 
@@ -684,7 +725,7 @@ void ScimBridgeAgentPanelListenerImpl::show_factory_menu (const vector<PanelFact
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.show_factory_menu (prepared_imcontext_id, menu);
+    scim_panel_client->show_factory_menu (menu);
 }
 
 
@@ -694,19 +735,29 @@ void ScimBridgeAgentPanelListenerImpl::show_help (const scim::String &string)
         scim_bridge_perrorln ("Panel client has not yet been prepared");
         return;
     }
-    scim_panel_client.show_help (prepared_imcontext_id, string);
+    scim_panel_client->show_help (string);
 }
 
 
-void ScimBridgeAgentPanelListenerImpl::register_input_context (scim_bridge_imcontext_id_t id, const String &factory_uuid)
+void ScimBridgeAgentPanelListenerImpl::register_input_context (const String &factory_uuid)
 {
-    scim_panel_client.register_input_context (id, factory_uuid);
+    if (prepared_imcontext_id == -1) {
+        scim_bridge_perrorln ("Panel client has not yet been prepared");
+        return;
+    }
+
+    scim_panel_client->register_input_context (factory_uuid);
 }
 
 
-void ScimBridgeAgentPanelListenerImpl::remove_input_context (scim_bridge_imcontext_id_t id)
+void ScimBridgeAgentPanelListenerImpl::remove_input_context ()
 {
-    scim_panel_client.remove_input_context (id);
+    if (prepared_imcontext_id == -1) {
+        scim_bridge_perrorln ("Panel client has not yet been prepared");
+        return;
+    }
+
+    scim_panel_client->remove_input_context ();
 }
 
 
