@@ -88,6 +88,9 @@ class ScimBridgeAgentImpl: public ScimBridgeAgent, public ScimBridgeAgentEventSe
         ScimBridgeAgentImpl (bool noexit_enabled);
         ~ScimBridgeAgentImpl ();
 
+        retval_t check_lockfile ();
+        retval_t create_lockfile (pid_t pid);
+
         void launch () throw (ScimBridgeAgentException);
 
         void push_event (const ScimBridgeAgentEvent *event);
@@ -227,7 +230,7 @@ ScimBridgeAgent *ScimBridgeAgent::alloc (bool noexit_enabled)
 
 
 ScimBridgeAgentImpl::ScimBridgeAgentImpl (bool noexit):
-quit_triggered (false), noexit_enabled (noexit), event_client_count (0), 
+quit_triggered (false), noexit_enabled (noexit), event_client_count (0),
 signal_listener (NULL), panel_listener (NULL), socket_listener (NULL)
 {
     pthread_mutex_init (&event_clients_mutex, NULL);
@@ -253,18 +256,18 @@ ScimBridgeAgentImpl::~ScimBridgeAgentImpl ()
     sem_trywait (&event_dispatcher_sem);
     sem_destroy (&event_dispatcher_sem);
 
-	if (signal_listener != NULL) {
-		signal_listener->close_event_client ();
-		delete signal_listener;
-	}
-	if (socket_listener != NULL) {
-		socket_listener->close_event_client ();
-		delete socket_listener;
-	}
-	if (panel_listener != NULL) {
-		panel_listener->close_event_client ();
-		delete panel_listener;
-	}
+    if (signal_listener != NULL) {
+        delete signal_listener;
+        signal_listener = NULL;
+    }
+    if (socket_listener != NULL) {
+        delete socket_listener;
+        socket_listener = NULL;
+    }
+    if (panel_listener != NULL) {
+        delete panel_listener;
+        panel_listener = NULL;
+    }
 }
 
 
@@ -327,7 +330,6 @@ void ScimBridgeAgentImpl::push_event (const ScimBridgeAgentEvent *event)
 {
     if (quit_triggered) return;
 
-    pthread_mutex_lock (&event_clients_mutex);
     events.push_back (event);
     int sem_value;
     sem_post (&event_dispatcher_sem);
@@ -352,9 +354,14 @@ const ScimBridgeAgentEvent *ScimBridgeAgentImpl::poll_event ()
                 sem_trywait (&event_dispatcher_sem);
                 sem_getvalue (&event_dispatcher_sem, &sem_value);
             } while (sem_value > 0);
-            pthread_mutex_unlock (&event_clients_mutex);
-            sem_wait (&event_dispatcher_sem);
-            pthread_mutex_lock (&event_clients_mutex);
+            if (quit_triggered) {
+                pthread_mutex_unlock (&event_clients_mutex);
+                return NULL;
+            } else {
+                pthread_mutex_unlock (&event_clients_mutex);
+                sem_wait (&event_dispatcher_sem);
+                pthread_mutex_lock (&event_clients_mutex);
+            }
         }
     }
     pthread_mutex_unlock (&event_clients_mutex);
@@ -398,21 +405,48 @@ void ScimBridgeAgentImpl::remove_event_client (ScimBridgeAgentEventClient *clien
 }
 
 
+retval_t ScimBridgeAgentImpl::create_lockfile (pid_t pid)
+{
+    if (panel_listener != NULL) {
+        return panel_listener->create_lockfile (pid);
+    } else {
+        return false;
+    }
+}
+
+
 void ScimBridgeAgentImpl::launch () throw (ScimBridgeAgentException)
 {
-	if (signal_listener->launch ()) {
-		throw ScimBridgeAgentException (String ("Failed to launch the signal listener"));
-	}
-
-	if (panel_listener->launch ()) {
-		throw ScimBridgeAgentException (String ("Failed to launch the panel listener"));
-	}
-
-	if (socket_listener->launch ()) {
-		throw ScimBridgeAgentException (String ("Failed to launch the socket listener"));
-	}
+    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "launch () @ ScimBridgeAgentImpl");
 
     if (pthread_create (&event_dispatcher_thread, NULL, &run_event_dispatcher, this)) throw ScimBridgeAgentException (String ("Cannot invoke the event dispatcher: ") + strerror (errno));
+
+    bool error = false;
+
+    if (!error && signal_listener->launch ()) {
+        scim_bridge_perrorln ("Failed to launch the signal listener");
+        push_event (new ScimBridgeAgentQuitEvent ());
+        error = true;
+    }
+
+    if (!error && panel_listener->launch ()) {
+        scim_bridge_perrorln ("Failed to launch the panel listener");
+        push_event (new ScimBridgeAgentQuitEvent ());
+        error = true;
+    }
+
+    ScimBridgeAgentIMContext::static_initialize (panel_listener, scim_backend);
+
+    ScimBridgeAgentIMContext::set_enabled_by_default (scim_config->read (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), ScimBridgeAgentIMContext::is_enabled_by_default ()));
+    ScimBridgeAgentIMContext::set_imengine_shared (scim_config->read (String (SCIM_CONFIG_FRONTEND_SHARED_INPUT_METHOD), ScimBridgeAgentIMContext::is_imengine_shared ()));
+    ScimBridgeAgentIMContext::set_on_the_spot_enabled_globally (scim_config->read (String (SCIM_CONFIG_FRONTEND_ON_THE_SPOT), ScimBridgeAgentIMContext::is_on_the_spot_enabled_globally ()));
+
+    if (!error && socket_listener->launch ()) {
+        scim_bridge_perrorln ("Failed to launch the socket listener");
+        push_event (new ScimBridgeAgentQuitEvent ());
+        error = true;
+    }
+
     if (pthread_join (event_dispatcher_thread, NULL)) throw ScimBridgeAgentException (String ("Cannot join the event dispatcher: ") + strerror (errno));
 }
 
@@ -526,7 +560,7 @@ retval_t ScimBridgeAgentImpl::initialize ()
     }
 
     // create backend
-    scim_backend = new CommonBackEnd (scim_config, user_imengine_vector.empty () ? user_imengine_vector : imengine_vector);
+    scim_backend = new CommonBackEnd (scim_config, !user_imengine_vector.empty () ? user_imengine_vector : imengine_vector);
 
     if (scim_backend.null ()) {
         scim_bridge_psyslogln (SYSLOG_CRITICAL, "Cannot create BackEnd Object!");
@@ -550,18 +584,13 @@ retval_t ScimBridgeAgentImpl::initialize ()
     add_event_client (panel_listener);
 
     socket_listener = ScimBridgeAgentSocketListener::alloc ();
-	if (socket_listener == NULL) return RETVAL_FAILED;
+    if (socket_listener == NULL) return RETVAL_FAILED;
     add_event_client (socket_listener);
 
     signal_listener = ScimBridgeAgentSignalListener::alloc ();
-	if (signal_listener == NULL) return RETVAL_FAILED;
+    if (signal_listener == NULL) return RETVAL_FAILED;
     add_event_client (signal_listener);
 
-    ScimBridgeAgentIMContext::static_initialize (panel_listener, scim_backend);
-
-    ScimBridgeAgentIMContext::set_on_the_spot_enabled_globally (scim_config->read (String (SCIM_CONFIG_FRONTEND_ON_THE_SPOT), ScimBridgeAgentIMContext::is_on_the_spot_enabled_globally ()));
-    ScimBridgeAgentIMContext::set_imengine_shared (scim_config->read (String (SCIM_CONFIG_FRONTEND_SHARED_INPUT_METHOD), ScimBridgeAgentIMContext::is_imengine_shared ()));
-
     scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 4, "ScimBridgeAgent is now ready");
     return RETVAL_SUCCEEDED;
 }
@@ -592,6 +621,7 @@ void ScimBridgeAgentImpl::finalize ()
     panel_listener = NULL;
 
     event_clients.clear ();
+    sem_post (&event_dispatcher_sem);
     pthread_mutex_unlock (&event_clients_mutex);
 
     ScimBridgeAgentIMContext::static_finalize ();
@@ -611,7 +641,7 @@ bool ScimBridgeAgentImpl::filter_hotkeys (ScimBridgeAgentIMContext *imcontext, c
     if (hotkey_action == SCIM_FRONTEND_HOTKEY_TRIGGER) {
         const bool new_status = !imcontext->is_enabled ();
         imcontext->set_enabled (new_status);
-        
+
         if (ScimBridgeAgentIMContext::is_imengine_shared ()) {
             ScimBridgeAgentIMContext::set_enabled_by_default (new_status);
             scim_config->write (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), new_status);
@@ -828,9 +858,9 @@ void ScimBridgeAgentImpl::slot_reload_config (const ConfigPointer &config)
     scim_imengine_hotkey_matcher.load_hotkeys (scim_config);
 
     if (!first_run) {
-        ScimBridgeAgentIMContext::set_on_the_spot_enabled_globally (scim_config->read (String (SCIM_CONFIG_FRONTEND_ON_THE_SPOT), ScimBridgeAgentIMContext::is_on_the_spot_enabled_globally ()));
-        ScimBridgeAgentIMContext::set_imengine_shared (scim_config->read (String (SCIM_CONFIG_FRONTEND_SHARED_INPUT_METHOD), ScimBridgeAgentIMContext::is_imengine_shared ()));
         ScimBridgeAgentIMContext::set_enabled_by_default (scim_config->read (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), ScimBridgeAgentIMContext::is_enabled_by_default ()));
+        ScimBridgeAgentIMContext::set_imengine_shared (scim_config->read (String (SCIM_CONFIG_FRONTEND_SHARED_INPUT_METHOD), ScimBridgeAgentIMContext::is_imengine_shared ()));
+        ScimBridgeAgentIMContext::set_on_the_spot_enabled_globally (scim_config->read (String (SCIM_CONFIG_FRONTEND_ON_THE_SPOT), ScimBridgeAgentIMContext::is_on_the_spot_enabled_globally ()));
     }
 
     // Get keyboard layout setting
@@ -1002,7 +1032,7 @@ void ScimBridgeAgentImpl::panel_change_factory (scim_bridge_imcontext_id_t imcon
         panel_listener->prepare (imcontext_id);
         imcontext->panel_change_factory (uuid);
         panel_listener->send ();
-	
+
         if (ScimBridgeAgentIMContext::is_imengine_shared ()) {
             scim_config->write (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), ScimBridgeAgentIMContext::is_enabled_by_default ());
         }
