@@ -77,7 +77,7 @@ static retval_t open_connection ()
     scim_bridge_exception_clear ();
 
     int i;
-    for (i = 0; i < 5; ++i) {
+    for (i = 0; i < 10; ++i) {
         socket_fd = socket (PF_UNIX, SOCK_STREAM, 0);
         if (socket_fd < 0) {
             scim_bridge_exception_occured (IO_EXCEPTION, "Failed to create the message socket: %s", strerror (errno));
@@ -89,7 +89,7 @@ static retval_t open_connection ()
         memset (&socket_addr, 0, sizeof (struct sockaddr_un));
         socket_addr.sun_family = AF_UNIX;
         strcpy (socket_addr.sun_path, scim_bridge_path_get_socket ());
-    
+
         if (connect (socket_fd, (struct sockaddr*)&socket_addr, sizeof (socket_addr.sun_family) + strlen (socket_addr.sun_path)) != 0) {
             if (i == 0 && launch_agent ()) {
                 scim_bridge_exception_push_stack ("open_connection ()");
@@ -120,7 +120,12 @@ static void connection_lost ()
 static retval_t received_message_unknown (const ScimBridgeMessage *message)
 {
     const char *message_header = scim_bridge_message_get_header (message);
-    scim_bridge_perrorln ("Unknown message received: %s", message_header);
+    scim_bridge_perror ("Unknown message received: %s", message_header);
+    int i;
+    for (i = 0; i < scim_bridge_message_get_argument_count (message); ++i) {
+        scim_bridge_perror (" %s", scim_bridge_message_get_argument (message, i));
+    }
+    scim_bridge_perrorln ("");
 
     return RETVAL_SUCCEEDED;
 }
@@ -970,7 +975,7 @@ retval_t scim_bridge_client_free_imcontext (ScimBridgeClientIMContext *imcontext
     }
 
     pending_imcontext_id = id;
-    
+
     while (pending_imcontext_id == id) {
         if (scim_bridge_client_read_and_dispatch ()) {
             scim_bridge_exception_push_stack ("scim_bridge_client_free_imcontext ()");
