@@ -116,6 +116,7 @@ static void connection_lost ()
 {
     ScimBridgeClientIMContext *seek;
     for (seek = first_imcontext; seek != NULL; seek = seek->next) {
+        scim_bridge_client_imcontext_set_preedit_shown (seek, FALSE);
         seek->id = -1;
     }
 }
@@ -217,18 +218,21 @@ scim_bridge_imcontext_id_t scim_bridge_client_imcontext_get_id (const ScimBridge
 
 void scim_bridge_client_imcontext_set_preedit_string (ScimBridgeClientIMContext *imcontext, const char *preedit_string)
 {
-    const size_t preedit_string_length = strlen (preedit_string);
+    size_t preedit_string_length;
+    if (preedit_string != NULL) {
+        preedit_string_length = strlen (preedit_string);
+    } else {
+        preedit_string_length = 0;
+    }
     if (imcontext->preedit_string_capacity <= preedit_string_length) {
         imcontext->preedit_string_capacity = preedit_string_length;
         free (imcontext->preedit_string);
         imcontext->preedit_string = malloc (sizeof (char) * (imcontext->preedit_string_capacity + 1));
     }
-    if (imcontext->preedit_string != NULL) {
+    if (preedit_string_length > 0) {
         strcpy (imcontext->preedit_string, preedit_string);
-        const size_t preedit_length = g_utf8_strlen (preedit_string, -1);
-        if (imcontext->preedit_cursor_position > preedit_length) imcontext->preedit_cursor_position = preedit_length;
     } else {
-        imcontext->preedit_cursor_position = 0;
+        imcontext->preedit_string[0] = '\0';
     }
 }
 
@@ -238,7 +242,8 @@ void scim_bridge_client_imcontext_set_preedit_shown (ScimBridgeClientIMContext *
     imcontext->preedit_shown = preedit_shown;
     if (!preedit_shown) {
         free (imcontext->preedit_string);
-        imcontext->preedit_string = NULL;
+        imcontext->preedit_string = malloc (sizeof (char));
+        imcontext->preedit_string[0] = '\0';
         imcontext->preedit_string_capacity = 0;
         imcontext->preedit_attribute_count = 0;
         imcontext->preedit_cursor_position = 0;
@@ -280,13 +285,22 @@ void scim_bridge_client_imcontext_update_preedit (const ScimBridgeClientIMContex
 
 void scim_bridge_client_imcontext_set_commit_string (ScimBridgeClientIMContext *imcontext, const char *commit_string)
 {
-    const size_t commit_string_length = strlen (commit_string);
+    size_t commit_string_length;
+    if (commit_string != NULL) {
+        commit_string_length = strlen (commit_string);
+    } else {
+        commit_string_length = 0;
+    }
     if (imcontext->commit_string_capacity <= commit_string_length) {
         imcontext->commit_string_capacity = commit_string_length;
         free (imcontext->commit_string);
         imcontext->commit_string = malloc (sizeof (char) * (imcontext->commit_string_capacity + 1));
     }
-    strcpy (imcontext->commit_string, commit_string);
+    if (commit_string_length > 0) {
+        strcpy (imcontext->commit_string, commit_string);
+    } else {
+        imcontext->commit_string[0] = '\0';
+    }
 }
 
 
@@ -463,14 +477,16 @@ void scim_bridge_client_imcontext_initialize (ScimBridgeClientIMContext *imconte
 {
     scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 5, "scim_bridge_client_imcontext_initialize");
 
-    imcontext->preedit_string = NULL;
+    imcontext->preedit_string = malloc (sizeof (char));
+    imcontext->preedit_string[0] = '\0';
     imcontext->preedit_string_capacity = 0;
 
     imcontext->preedit_attributes = NULL;
     imcontext->preedit_attribute_count = 0;
     imcontext->preedit_attribute_capacity = 0;
 
-    imcontext->commit_string = NULL;
+    imcontext->commit_string = malloc (sizeof (char));
+    imcontext->commit_string[0] = '\0';
     imcontext->commit_string_capacity = 0;
 
     imcontext->client_window = NULL;
@@ -529,7 +545,7 @@ void scim_bridge_client_imcontext_finalize (GObject *object)
     for (i = 0; i < imcontext->preedit_attribute_capacity; ++i) {
         scim_bridge_free_attribute (imcontext->preedit_attributes[i]);
     }
-	free (imcontext->preedit_attributes);
+    free (imcontext->preedit_attributes);
     imcontext->preedit_attributes = NULL;
     imcontext->preedit_attribute_count = 0;
     imcontext->preedit_attribute_capacity = 0;
@@ -604,6 +620,8 @@ void scim_bridge_client_imcontext_reset (GtkIMContext *context)
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
+    if (imcontext != focused_imcontext) return;
+
     if (scim_bridge_client_is_active () && imcontext != NULL) {
         if (imcontext->flags & FLAG_PENDING) {
             imcontext->flags |= FLAG_RESET;
@@ -623,27 +641,29 @@ void scim_bridge_client_imcontext_get_preedit_string (GtkIMContext *context, gch
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
     if (scim_bridge_client_is_active () && imcontext != NULL && imcontext->preedit_shown) {
+        const size_t preedit_string_length = strlen (imcontext->preedit_string);
+        
         if (str) {
-            if (imcontext->preedit_string != NULL && strlen (imcontext->preedit_string) > 0) {
+            if (preedit_string_length > 0) {
                 *str = g_strdup (imcontext->preedit_string);
             } else {
                 *str = g_strdup ("");
             }
         }
-        if (cursor_pos) *cursor_pos = imcontext->preedit_cursor_position;
+        if (cursor_pos) {
+            const size_t preedit_length = g_utf8_strlen (imcontext->preedit_string, -1);
+            if (imcontext->preedit_cursor_position > preedit_length) {
+                *cursor_pos = preedit_length;
+            } else {
+                *cursor_pos = imcontext->preedit_cursor_position;
+            }
+        }
 
         if (pango_attrs) {
             *pango_attrs = pango_attr_list_new ();
 
             gboolean underline_exists = FALSE;
 
-            size_t preedit_string_length;
-            if (imcontext->preedit_string != NULL) {
-                preedit_string_length = strlen (imcontext->preedit_string);
-            } else {
-                preedit_string_length = 0;
-            }
-
             int i;
             for (i = 0; i < imcontext->preedit_attribute_count; ++i) {
                 const ScimBridgeAttribute *attr = imcontext->preedit_attributes[i];
@@ -730,7 +750,7 @@ void scim_bridge_client_imcontext_focus_in (GtkIMContext *context)
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
-    if (focused_imcontext != NULL && focused_imcontext != imcontext) scim_bridge_client_imcontext_focus_out (GTK_IM_CONTEXT (focused_imcontext));
+    if (focused_imcontext != NULL) scim_bridge_client_imcontext_focus_out (GTK_IM_CONTEXT (focused_imcontext));
 
     focused_imcontext = imcontext;
 
@@ -755,6 +775,10 @@ void scim_bridge_client_imcontext_focus_out (GtkIMContext *context)
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
     focused_imcontext = imcontext;
+    if (imcontext->preedit_shown) {
+        scim_bridge_client_imcontext_set_preedit_shown (imcontext, FALSE);
+        scim_bridge_client_imcontext_update_preedit (imcontext);
+    }
     if (scim_bridge_client_is_active () && imcontext != NULL) {
 
         if (scim_bridge_client_focus_changed (imcontext, FALSE)) {
@@ -782,6 +806,7 @@ void scim_bridge_client_imcontext_set_client_window (GtkIMContext *context, GdkW
         if (imcontext->client_window != NULL) g_object_unref (imcontext->client_window);
         if (new_window != NULL) g_object_ref (new_window);
         imcontext->client_window = new_window;
+        gdk_window_get_origin (imcontext->client_window, &imcontext->window_x, &imcontext->window_y);
     }
 }
 
