@@ -116,7 +116,7 @@ static void static_initialize ()
     if (display_number < 0 || screen_number < 0) scim_bridge_perrorln ("Invalid environmental variable: DISPLAY = %s", display_name == NULL ? "NULL":display_name);
 
     if (display_name != NULL) {
-        const uid_t uid = getuid ();
+        const uid_t uid = geteuid ();
         const size_t lockfile_path_length = snprintf (NULL, 0, "%s/%s-%d-%d-%d", LOCKFILE_DIR, LOCKFILE_NAME, display_number, screen_number, uid);
 
         lockfile_path = malloc (sizeof (char) * (lockfile_path_length + 1));
@@ -167,7 +167,7 @@ void scim_bridge_free_display (ScimBridgeDisplay *display)
 
 void scim_bridge_copy_display (ScimBridgeDisplay *dst, const ScimBridgeDisplay *src)
 {
-    const size_t display_name_str_length = strlen (src->name);    
+    const size_t display_name_str_length = strlen (src->name);
     free (dst->name);
     dst->name = malloc (sizeof (char) * (display_name_str_length + 1));
     strcpy (dst->name, src->name);
@@ -179,13 +179,13 @@ void scim_bridge_copy_display (ScimBridgeDisplay *dst, const ScimBridgeDisplay *
 void scim_bridge_display_fetch_current (ScimBridgeDisplay *display)
 {
     pthread_once (&static_initializer, &static_initialize);
-    
+
     const size_t display_name_str_length = strlen (display_name);
 
     free (display->name);
     display->name = malloc (sizeof (char) * (display_name_str_length + 1));
     strcpy (display->name, display_name);
-    
+
     display->display_number = display_number;
     display->screen_number = screen_number;
 }
